CREATE OR REPLACE TRIGGER TRG_USER_ERC_TO_OUT
  BEFORE INSERT OR UPDATE OR DELETE ON USER_ERC
  FOR EACH ROW
declare
  var_operation       EVENT_OUT_USER.operation%type;
  var_op_moveou       EVENT_OUT_USER.operation%type; 
  var_ext_table       EVENT_OUT_USER.erc_id%type;
  var_code            USER_ERC.pm_code%type;
  var_old_code        USER_ERC.pm_code%type;
  var_ou              USER_ERC.ou%type;
  var_old_ou          USER_ERC.ou%type;
  code_oper           varchar2(64);
  counter             number(16);
begin 
	if (trim(upper(user)) <> trim(upper('IDEAS'))) then
	return;
	end if;
	
	select count(1) into counter from tmp_codoperation;

    if ( counter = 0 ) then
      RAISE_APPLICATION_ERROR ( -20200, 'NO OPERATION CODE FOUND IN TMP_CODOPERATION TABLE');
    elsif ( counter = 1 ) then
      select code into code_oper from tmp_codoperation;
    elsif  ( counter > 1 ) then
      RAISE_APPLICATION_ERROR ( -20201, 'TOO MANY OPERATION CODE FOUND IN TMP_CODOPERATION TABLE');
    end if;
	
	-- Attributes mapping for USER_ERC
	-- new.PM_CODE ---> OBJ_CODE
	-- old.PM_CODE ---> attr1
	-- new.OU ---> attr2
	-- old.OU ---> attr3

	if (inserting) then
		var_operation    := 1;
		var_ext_table    := :new.id;
		var_code         := :new.pm_code;
		var_ou           := :new.ou;
    if (var_code is null) then 
      -- skip
      return;
    end if;
	elsif (updating) then
		var_operation := 2;
		var_op_moveou := 12;
		if (:new.pm_code is not null and :old.pm_code is null) then 
			-- if pm_code has been set for the first time it is a new user creation
			var_operation := 1;
		end if;    
		var_ext_table    := :new.id;
		var_code         := :new.pm_code;
		var_ou           := :new.ou;
		if (:new.pm_code <> :old.pm_code) then
			var_old_code := :old.pm_code;
		end if;
		if (:new.ou <> :old.ou) then 
			var_old_ou     := :old.ou;
		end if;     
	elsif (deleting) then
		var_operation := 3;
		var_ext_table := :old.id;
		var_ou       := :old.ou;
		var_code     := :old.pm_code;
	end if;

	-- if OU changed generate a move user event before, operation=12
	if ( (updating) and 
		 (( :old.ou is null and :new.ou is not null ) or ( :old.ou is not null and :new.ou is null ) or ( :old.ou != :new.ou ))
	   )
	then
	  insert into EVENT_OUT_USER
		(id, erc_id, erc_name, operation, cod_operation, status, last_mod_time, last_mod_user, obj_code, ext_attr1, ext_attr2, ext_attr3)
	  values
		(EVENT_OUT_USER_SEQ.nextval, var_ext_table, 'user_erc', var_op_moveou, code_oper, 0, sysdate, 'trg_user_to_out', var_code, null, var_ou, var_old_ou);
	end if;

	-- insert row
	insert into EVENT_OUT_USER
		(id, erc_id, erc_name, operation, cod_operation, status, last_mod_time, last_mod_user, obj_code, ext_attr1, ext_attr2, ext_attr3)
	values
		(EVENT_OUT_USER_SEQ.nextval, var_ext_table, 'user_erc', var_operation, code_oper, 0, sysdate, 'trg_user_to_out', var_code, var_old_code, var_ou, var_old_ou);

end TRG_USER_ERC_TO_OUT;
/